import fg from 'fast-glob';
import { setupChai } from './integrations/chai/setup';
import { clearContext, defaultSuite } from './suite';
import { context } from './context';
import { afterEachHook, afterFileHook, afterAllHook, afterSuiteHook, beforeEachHook, beforeFileHook, beforeAllHook, beforeSuiteHook } from './hooks';
import { DefaultReporter } from './reporters/default';
import { defaultIncludes, defaultExcludes } from './constants';
export async function runTask(task, ctx) {
    var _a, _b;
    const { reporter } = ctx;
    await ((_a = reporter.onTaskBegin) === null || _a === void 0 ? void 0 : _a.call(reporter, task, ctx));
    if (task.mode === 'run') {
        await beforeEachHook.fire(task);
        try {
            await task.fn();
            task.state = 'pass';
        }
        catch (e) {
            task.state = 'fail';
            task.error = e;
        }
        await afterEachHook.fire(task);
    }
    await ((_b = reporter.onTaskEnd) === null || _b === void 0 ? void 0 : _b.call(reporter, task, ctx));
}
export async function collectFiles(paths) {
    const files = [];
    for (const filepath of paths) {
        const file = {
            filepath,
            suites: [],
            collected: false,
        };
        clearContext();
        try {
            await import(filepath);
            const collectors = [defaultSuite, ...context.suites];
            for (const c of collectors) {
                context.currentSuite = c;
                file.suites.push(await c.collect(file));
            }
            file.collected = true;
        }
        catch (e) {
            file.error = e;
            file.collected = false;
            process.exitCode = 1;
        }
        files.push(file);
    }
    const allSuites = files.reduce((suites, file) => suites.concat(file.suites), []);
    interpretOnlyMode(allSuites);
    allSuites.forEach((i) => {
        if (i.mode === 'skip')
            i.tasks.forEach(t => t.mode === 'run' && (t.state = 'skip'));
        else
            interpretOnlyMode(i.tasks);
    });
    return files;
}
/**
 * If any items been marked as `only`, mark all other items as `skip`.
 */
function interpretOnlyMode(items) {
    if (items.some(i => i.mode === 'only')) {
        items.forEach((i) => {
            if (i.mode === 'run')
                i.mode = 'skip';
            else if (i.mode === 'only')
                i.mode = 'run';
        });
    }
}
export async function runFile(file, ctx) {
    var _a, _b, _c, _d;
    const { reporter } = ctx;
    const runableSuites = file.suites.filter(i => i.mode === 'run');
    if (runableSuites.length === 0)
        return;
    await ((_a = reporter.onFileBegin) === null || _a === void 0 ? void 0 : _a.call(reporter, file, ctx));
    await beforeFileHook.fire(file);
    for (const suite of file.suites) {
        await ((_b = reporter.onSuiteBegin) === null || _b === void 0 ? void 0 : _b.call(reporter, suite, ctx));
        await beforeSuiteHook.fire(suite);
        for (const t of suite.tasks)
            await runTask(t, ctx);
        await afterSuiteHook.fire(suite);
        await ((_c = reporter.onSuiteEnd) === null || _c === void 0 ? void 0 : _c.call(reporter, suite, ctx));
    }
    await afterFileHook.fire(file);
    await ((_d = reporter.onFileEnd) === null || _d === void 0 ? void 0 : _d.call(reporter, file, ctx));
}
export async function run(config) {
    var _a, _b, _c, _d;
    // setup chai
    await setupChai(config);
    // collect files
    let paths = await fg(config.includes || defaultIncludes, {
        absolute: true,
        cwd: config.rootDir,
        ignore: config.excludes || defaultExcludes,
    });
    if ((_a = config.nameFilters) === null || _a === void 0 ? void 0 : _a.length)
        paths = paths.filter(i => config.nameFilters.some(f => i.includes(f)));
    if (!paths.length) {
        console.error('No test files found');
        process.exitCode = 1;
        return;
    }
    const reporter = new DefaultReporter();
    await ((_b = reporter.onStart) === null || _b === void 0 ? void 0 : _b.call(reporter, config));
    if (config.global)
        (await import('./global')).registerApiGlobally();
    if (config.jsdom)
        (await import('./integrations/jsdom')).setupJSDOM(globalThis);
    const files = await collectFiles(paths);
    const ctx = {
        files,
        config,
        reporter,
    };
    await ((_c = reporter.onCollected) === null || _c === void 0 ? void 0 : _c.call(reporter, ctx));
    await beforeAllHook.fire();
    for (const file of files)
        await runFile(file, ctx);
    await afterAllHook.fire();
    await ((_d = reporter.onFinished) === null || _d === void 0 ? void 0 : _d.call(reporter, ctx));
}
